/*
 * BinCramerPhiS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Oct-14
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * CRAMER PHI MEASURE OF ASSOCIATION
 * 
 * <br>http://www.psych.cornell.edu/Darlington/crosstab/table3.htm
 * <br>The Cramer phi and phi2 are simple transformations of the familiar 
 * Pearson chi-square statistic for testing association in a frequency table. 
 * This statistic uses the familiar formula: 
 * <br>chi-square = SUM((o-e)^2/e)
 * <br>where e = row total * column total / N
 * <br>phi = sqrt(chisquare/(N(MRC-1)))
 * <br>In 2x2 tables the Cramer phi reduces to the absolute value of the 
 * familiar phi-coefficient, which in turn equals the Pearson correlation 
 * between the row and column variables when both variables are coded 
 * numerically such as 0-1 or 1-2.
 * <br>Similarity [0, 1].
 * <br>Usage: for binary data.
 * 
 * @author  Martin Hassel
 * @version 2004-Oct-14
 */
public class BinCramerPhiS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		int a = 0; /* present in both samples */
		int b = 0; /* present in sample i, absent in sample j */
		int c = 0; /* present in sample j, absent in sample i */
		int d = 0; /* absent in both samples */
		double ea = 0; /* expected to be present in both samples */
		double eb = 0; /* expected to be present in sample i, absent in sample j */
		double ec = 0; /* expected to be present in sample j, absent in sample i */
		double ed = 0; /* expected to be absent in both samples */
		int mrc;       /* minimum of rows and columns of the contongency table, here 2 */
		double chisq;  /* chi-square statistic */
		double sim_matrix[][]; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		/* Begin calculations */
		mrc = 2;
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				a = b = c = d = 0;
				for(int column = 0; column < columns; ++column) {
					if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 1)
						a++;
					else if(data_matrix[row1][column] == 1 && data_matrix[row2][column] == 0)
						b++;
					else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 1)
						c++;
					else if(data_matrix[row1][column] == 0 && data_matrix[row2][column] == 0)
						d++;
				}
				ea = (a+b)*(a+c)/(double)columns;
				eb = (a+b)*(b+d)/(double)columns;
				ec = (c+d)*(a+c)/(double)columns;
				ed = (c+d)*(b+d)/(double)columns;
				chisq = 
					(a-ea)*(a-ea) / (double)ea+(b-eb)*(b-eb) / (double)eb+(c-ec)*(c-ec) / (double)ec+(d-ed)*(d-ed) / (double)ed;
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
					Math.sqrt(chisq/(double)(columns*(mrc-1)));
			}
		}
		return sim_matrix;
	}
}
