/*
 * CosineD_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-02
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * COSINE DISTANCE MEASURE
 * 
 * <br>Originally written by P. Cejchan, 1999/08/13. Algorithm taken from:
 * Carbonell, J.G.& al. 1997 "Translingual Information Retrieval: A comparative
 * evaluation. IJCAI'97". See also Salton, G. 1989 "Automatic text processing:
 * The transformation, Analysis, and retrieval of information by computer".
 * Addison-Wesley, Reading, Pennsylvania. (Jongman, et. al., 1995, page
 * 178)--"More emphasis is given to qualitative aspects by not considering a
 * site as point but as a vector. Understandably, the direction of this vector
 * tells us something about the relative abundances of species. The similarity
 * of two sites can be expressed as some function of the angle between the
 * vector of these sites. Quite common is the use of the cosine (or Ochiai
 * coefficient):
 * cos=OS=sigma(k)Y(ki)Y(kj)/sqrt{[sigma(k)(Y(ki)^2)][sigma(k)(Y(kj))^2)]}" 
 * <-- this is obviously for distance between columns, not rows (P.C).
 * 
 * @author Martin Hassel
 * @version 2004-Sep-03
 */
public class CosineD_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double x, y, numerator, sum1, sum2;
		double[][] dist_matrix; /* matrix to contain the distance */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate distance matrix */
		dist_matrix = new double[rows][rows];
		
		/* Set diagonal to 0 */
		for(int row = 0; row < rows; ++row)
			dist_matrix[row][row] = 0.0;
		
		/* Begin calculations */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				numerator = sum1 = sum2 = 0.0;
				if(data_matrix[row1].length != data_matrix[row2].length)
					dist_matrix[row1][row2] = 0;
				else if(data_matrix[row1] == null || data_matrix[row2] == null)
					dist_matrix[row1][row2] = 0;
				else
					for(int column = 0; column < columns; ++column) {
						x = data_matrix[row1][column];
						y = data_matrix[row2][column];
						numerator += x * y;
						sum1 += x * x;
						sum2 += y * y;
					}
				if(sum1 == 0 || sum2 == 0)
					dist_matrix[row1][row2] = 0;
				dist_matrix[row1][row2] = dist_matrix[row2][row1] = 
					numerator / Math.sqrt(sum1 * sum2);
			}
		}
		return dist_matrix;
	}
}