/*
 * BinPearsonCorrS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Oct-01
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * PEARSON CORRELATION COEFFICIENT 
 * 
 * <br>http://stat.tamu.edu/stat30x/notes/node39.html
 * <br>similarity [-1, 1]
 * <br>Usage: for interval & count(?) data.
 * <br>Sample data: 
 * <br>             X 1 2 3
 * <br>             Y 2 5 6
 * <br>Result:
 * <br>            1.0      0.9607689228305224
 * <br>            0.9607689228305224      1.0
 * <br>Source: HyperStat Online Contents, Chapter3, Describing Bivariate Data
 * http://davidmlane.com/hyperstat/desc_biv.html
 * 
 * @author  Martin Hassel
 * @version 2004-Oct-01
 */
public class PearsonCorrS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double sim_matrix[][]; /* matrix to contain the similarity */
		double xmean, ymean, sxx, syy, sxy;
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		/* Begin calculations */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < columns; ++row2) {
				xmean = ymean = 0.0;
				for(int column = 0; column < columns; ++column) {
					xmean += data_matrix[row1][column];
					ymean += data_matrix[row2][column];
				}
				xmean /= columns;
				ymean /= columns;
				sxx = syy = sxy = 0.0;
				for(int column = 0; column < columns; ++column) {
					sxx += data_matrix[row1][column]*data_matrix[row1][column];
					syy += data_matrix[row2][column]*data_matrix[row2][column];
					sxy += (data_matrix[row1][column])*(data_matrix[row2][column]);
				}
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
					(sxy-xmean*ymean*columns)/Math.sqrt((sxx-xmean*xmean*columns)*(syy-ymean*ymean*columns));
			}
		}
		return sim_matrix;
	}
}
